@@ -480,6 +480,7 @@ const ng_parser_t ng_parser_native = {
 	.dict_add_string = bencode_dictionary_add_string,
 	.dict_add_str = bencode_dictionary_add_str,
 	.dict_add_str_dup = bencode_dictionary_add_str_dup,
+	.dict_add_str_dup_dup = bencode_dictionary_add_str_dup_dup,
 	.dict_add_int = bencode_dictionary_add_integer,
 	.dict_add_dict = __bencode_dictionary_add_dictionary,
 	.dict_add_dict_dup = __bencode_dictionary_add_dictionary_dup,
@@ -516,6 +517,7 @@ const ng_parser_t ng_parser_json = {
 	.dict_add_string = json_dict_add_string,
 	.dict_add_str = json_dict_add_str,
 	.dict_add_str_dup = json_dict_add_str,
+	.dict_add_str_dup_dup = json_dict_add_str,
 	.dict_add_int = json_dict_add_int,
 	.dict_add_dict = json_dict_add_dict,
 	.dict_add_dict_dup = json_dict_add_dict,
