@@ -18,8 +18,6 @@
 
 #include "xt_RTPENGINE.h"
 
-__thread GString *crypto_debug_string;
-
 static int aes_cm_encrypt_rtp(struct crypto_context *, struct rtp_header *, str *, uint32_t);
 static int aes_cm_encrypt_rtcp(struct crypto_context *, struct rtcp_packet *, str *, uint32_t);
 static int aes_gcm_encrypt_rtp(struct crypto_context *, struct rtp_header *, str *, uint32_t);
@@ -446,7 +444,7 @@ static void prf_n(str *out, const unsigned char *key, const EVP_CIPHER *ciph, co
 	memcpy(iv, x, 14);
 	/* iv[14] = iv[15] = 0;   := x << 16 */
 	ZERO(in); /* outputs the key stream */
-	str_init_len(&in_s, (void *) in, out->len > 16 ? 32 : 16);
+	in_s = STR_LEN(in, out->len > 16 ? 32 : 16);
 	aes_ctr_no_ctx(o, &in_s, key, ciph, iv);
 
 	memcpy(out->s, o, out->len);
@@ -860,7 +858,7 @@ static int aes_cm_session_key_init(struct crypto_context *c) {
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
 	c->session_key_ctx[0] = EVP_CIPHER_CTX_new();
 #else
-	c->session_key_ctx[0] = g_slice_alloc(sizeof(EVP_CIPHER_CTX));
+	c->session_key_ctx[0] = g_new(EVP_CIPHER_CTX, 1);
 	EVP_CIPHER_CTX_init(c->session_key_ctx[0]);
 #endif
 	EVP_EncryptInit_ex(c->session_key_ctx[0], c->params.crypto_suite->aes_evp, NULL,
@@ -874,7 +872,7 @@ static int aes_gcm_session_key_init(struct crypto_context *c) {
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
 	c->session_key_ctx[0] = EVP_CIPHER_CTX_new();
 #else
-	c->session_key_ctx[0] = g_slice_alloc(sizeof(EVP_CIPHER_CTX));
+	c->session_key_ctx[0] = g_new(EVP_CIPHER_CTX, 1);
 	EVP_CIPHER_CTX_init(c->session_key_ctx[0]);
 #endif
 	return 0;
@@ -903,7 +901,7 @@ static int aes_f8_session_key_init(struct crypto_context *c) {
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
 	c->session_key_ctx[1] = EVP_CIPHER_CTX_new();
 #else
-	c->session_key_ctx[1] = g_slice_alloc(sizeof(EVP_CIPHER_CTX));
+	c->session_key_ctx[1] = g_new(EVP_CIPHER_CTX, 1);
 	EVP_CIPHER_CTX_init(c->session_key_ctx[1]);
 #endif
 	EVP_EncryptInit_ex(c->session_key_ctx[1], EVP_aes_128_ecb(), NULL, m, NULL);
@@ -924,7 +922,7 @@ static int evp_session_key_cleanup(struct crypto_context *c) {
 		EVP_CIPHER_CTX_free(c->session_key_ctx[i]);
 #else
 		EVP_CIPHER_CTX_cleanup(c->session_key_ctx[i]);
-		g_slice_free1(sizeof(EVP_CIPHER_CTX), c->session_key_ctx[i]);
+		g_free(c->session_key_ctx[i]);
 #endif
 		c->session_key_ctx[i] = NULL;
 	}
@@ -997,7 +995,7 @@ void crypto_init_main(void) {
 	for (unsigned int i = 0; i < num_crypto_suites; i++) {
 		cs = &__crypto_suites[i];
 		cs->idx = i;
-		str_init(&cs->name_str, (char *) cs->name);
+		cs->name_str = STR(cs->name);
 		switch(cs->master_key_len) {
 		case 16:
 			cs->aes_evp = EVP_aes_128_ecb();
@@ -1012,9 +1010,9 @@ void crypto_init_main(void) {
 	}
 }
 
-void __crypto_debug_printf(const char *fmt, ...) {
+void __crypto_debug_printf(crypto_debug_string *gs, const char *fmt, ...) {
 	va_list va;
 	va_start(va, fmt);
-	g_string_append_vprintf(crypto_debug_string, fmt, va);
+	g_string_append_vprintf(gs, fmt, va);
 	va_end(va);
 }
