@@ -19,7 +19,6 @@
 #include "xt_RTPENGINE.h"
 
 #define PREFIX "/proc/rtpengine"
-#define MMAP_PAGE_SIZE (4096 * 16)
 
 struct kernel_interface kernel;
 
@@ -58,13 +57,38 @@ static bool kernel_delete_table(unsigned int id) {
 	return kernel_action_table("del", id);
 }
 
-static void *kernel_alloc(size_t len) {
-	void *b = mmap(NULL, len, PROT_READ | PROT_WRITE, MAP_SHARED, kernel.fd, 0);
-	assert(b != NULL && b != MAP_FAILED);
-	return b;
+static void *kernel_alloc(void) {
+	// Since we can't really request memory at a specific location that we know
+	// will be correctly aligned, request twice as much, which we know must be
+	// enough to contain at least one correctly aligned block. This may seem like
+	// a waste, but the extra pages won't ever be used, and so usually won't even
+	// be mapped.
+	void *b = mmap(NULL, BUFFERPOOL_SHARD_SIZE * 2, PROT_READ | PROT_WRITE, MAP_SHARED, kernel.fd, 0);
+
+	if (b == NULL || b == MAP_FAILED) {
+		ilog(LOG_CRIT, "Failed to allocate shared kernel memory: %s", strerror(errno));
+		abort();
+	}
+
+	// find the aligned block
+	void *aligned = (void *) (((intptr_t) b + BUFFERPOOL_SHARD_SIZE - 1) & BUFFERPOOL_TOP_MASK);
+
+	// place a pointer to the real beginning of the block just past the end, so we
+	// know what to free
+	void **back_ptr = aligned + BUFFERPOOL_SHARD_SIZE;
+	// make sure there is enough extra space to store our back pointer (there should be, unless
+	// our page size is really tiny)
+	assert((void *) back_ptr + sizeof(void *) < b + BUFFERPOOL_SHARD_SIZE * 2);
+
+	*back_ptr = b;
+
+	return aligned;
 }
-static void kernel_free(void *p, size_t len) {
-	munmap(p, len);
+static void kernel_free(void *p) {
+	// restore saved pointer to read beginning of the block
+	void **back_ptr = p + BUFFERPOOL_SHARD_SIZE;
+	p = *back_ptr;
+	munmap(p, BUFFERPOOL_SHARD_SIZE * 2);
 }
 
 static int kernel_open_table(unsigned int id) {
@@ -144,7 +168,7 @@ bool kernel_setup_table(unsigned int id) {
 	kernel.table = id;
 	kernel.is_open = true;
 
-	shm_bufferpool = bufferpool_new2(kernel_alloc, kernel_free, MMAP_PAGE_SIZE);
+	shm_bufferpool = bufferpool_new(kernel_alloc, kernel_free);
 
 	return true;
 }
