@@ -105,7 +105,7 @@ static int t38_gateway_handler(t38_core_state_t *stat, void *user_data, const ui
 	g_string_append_len(s, (void *) &seq, 2);
 
 	// add primary IFP packet
-	str buf = STR_INIT_LEN(b, len);
+	str buf = STR_LEN(b, len);
 	__add_udptl(s, &buf);
 
 	// add error correction packets
@@ -238,8 +238,7 @@ static int t38_gateway_handler(t38_core_state_t *stat, void *user_data, const ui
 	return 0;
 }
 
-void __t38_gateway_free(void *p) {
-	struct t38_gateway *tg = p;
+void __t38_gateway_free(struct t38_gateway *tg) {
 	ilog(LOG_DEBUG, "Destroying T.38 gateway");
 	if (tg->gw)
 		t38_gateway_free(tg->gw);
@@ -275,8 +274,8 @@ static bool t38_pcm_player(struct media_player *mp) {
 	if (num <= 0) {
 		ilog(LOG_DEBUG, "No T.38 PCM samples generated");
 		// use a fixed interval of 10 ms
-		timeval_add_usec(&mp->next_run, 10000);
-		timerthread_obj_schedule_abs(&mp->tt_obj, &mp->next_run);
+		mp->next_run += 10000;
+		timerthread_obj_schedule_abs(&mp->tt_obj, mp->next_run);
 		mutex_unlock(&tg->lock);
 		return false;
 	}
@@ -290,7 +289,7 @@ static bool t38_pcm_player(struct media_player *mp) {
 	tg->pts += num;
 
 	// handle fill-in
-	if (timeval_diff(&rtpe_now, &tg->last_rx_ts) > 30000) {
+	if (rtpe_now - tg->last_rx_ts > 30000) {
 		ilog(LOG_DEBUG, "Adding T.38 fill-in samples");
 		t38_gateway_rx_fillin(tg->gw, 80);
 	}
@@ -308,7 +307,7 @@ static bool t38_pcm_player(struct media_player *mp) {
 static void __udptl_packet_free(struct udptl_packet *p) {
 	if (p->s)
 		free(p->s);
-	g_slice_free1(sizeof(*p), p);
+	g_free(p);
 }
 
 
@@ -341,8 +340,10 @@ static int span_log_level_map(int level) {
 	return level;
 }
 
-void t38_insert_media_attributes(GString *gs, union sdp_attr_print_arg a, const sdp_ng_flags *flags) {
-	struct t38_gateway *tg = a.cm->t38_gateway;
+static void t38_insert_media_attributes(GString *gs, struct call_media *media, struct call_media *source_media,
+		const sdp_ng_flags *flags)
+{
+	struct t38_gateway *tg = media->t38_gateway;
 	if (!tg)
 		return;
 
@@ -390,7 +391,7 @@ int t38_gateway_pair(struct call_media *t38_media, struct call_media *pcm_media,
 	ilog(LOG_DEBUG, "Creating new T.38 gateway");
 
 	// create and init new
-	struct t38_gateway *tg = obj_alloc0("t38_gateway", sizeof(*tg), __t38_gateway_free);
+	__auto_type tg = obj_alloc0(struct t38_gateway, __t38_gateway_free);
 
 	tg->t38_media = t38_media;
 	tg->pcm_media = pcm_media;
@@ -400,7 +401,7 @@ int t38_gateway_pair(struct call_media *t38_media, struct call_media *pcm_media,
 	tg->options = opts;
 
 	tg->pcm_pt.payload_type = -1;
-	str_init(&tg->pcm_pt.encoding, "PCM-S16LE");
+	tg->pcm_pt.encoding = STR("X-L16");
 	tg->pcm_pt.encoding_with_params = tg->pcm_pt.encoding;
 	tg->pcm_pt.clock_rate = 8000;
 	tg->pcm_pt.channels = 1;
@@ -416,7 +417,8 @@ int t38_gateway_pair(struct call_media *t38_media, struct call_media *pcm_media,
 		goto err;
 
 	media_player_new(&tg->pcm_player, pcm_media->monologue,
-			pcm_media->streams.length ? pcm_media->streams.head->data->ssrc_out[0] : NULL);
+			call_get_first_ssrc(&pcm_media->ssrc_hash_out),
+			NULL);
 	// even though we call media_player_set_media() here, we need to call it again in
 	// t38_gateway_start because our sink might not have any streams added here yet,
 	// leaving the media_player setup incomplete
@@ -470,18 +472,18 @@ err:
 
 
 // call is locked in W
-void t38_gateway_start(struct t38_gateway *tg) {
+void t38_gateway_start(struct t38_gateway *tg, str_case_value_ht codec_set) {
 	if (!tg)
 		return;
 
 	// set up our player first
 	media_player_set_media(tg->pcm_player, tg->pcm_media);
-	if (media_player_setup(tg->pcm_player, &tg->pcm_pt, NULL))
+	if (media_player_setup(tg->pcm_player, &tg->pcm_pt, NULL, codec_set))
 		return;
 
 	// now start our player if we can or should
 	// already running?
-	if (tg->pcm_player->next_run.tv_sec)
+	if (tg->pcm_player->next_run)
 		return;
 
 	// only start our player only if we can send both ways
@@ -504,7 +506,7 @@ void t38_gateway_start(struct t38_gateway *tg) {
 
 	// start off PCM player
 	tg->pcm_player->next_run = rtpe_now;
-	timerthread_obj_schedule_abs(&tg->pcm_player->tt_obj, &tg->pcm_player->next_run);
+	timerthread_obj_schedule_abs(&tg->pcm_player->tt_obj, tg->pcm_player->next_run);
 }
 
 
@@ -567,7 +569,7 @@ static int __get_udptl(str *piece, str *s) {
 
 
 static struct udptl_packet *__make_udptl_packet(const str *piece, uint16_t seq) {
-	struct udptl_packet *up = g_slice_alloc0(sizeof(*up));
+	struct udptl_packet *up = g_new0(__typeof(*up), 1);
 	up->p.seq = seq;
 	up->s = str_dup(piece);
 	return up;
@@ -734,7 +736,7 @@ int t38_gateway_input_udptl(struct t38_gateway *tg, const str *buf) {
 							"packet with seq %i from FEC",
 							seq_fec);
 
-					str rec_str = STR_INIT_GS(rec_s);
+					str rec_str = STR_GS(rec_s);
 					__fec_save(tg, &rec_str, seq_fec);
 					up = __make_udptl_packet(&rec_str, seq_fec);
 					packet_sequencer_insert(&tg->sequencer, &up->p);
