@@ -35,14 +35,16 @@ struct dtls_cert {
 	GQueue fingerprints;
 	EVP_PKEY *pkey;
 	X509 *x509;
-	time_t expires;
+	int64_t expires_us;
 };
 
 struct dtls_connection {
 	SSL_CTX *ssl_ctx;
 	SSL *ssl;
 	BIO *r_bio, *w_bio;
-	void *ptr;
+	struct packet_stream *ps;
+	endpoint_t fsin;
+	stream_fd *sfd;
 	unsigned char tls_id[16];
 	unsigned int init:1,
 	             active:1,
