@@ -43,6 +43,7 @@ enum subscribe_action {
 struct redis {
 	endpoint_t	endpoint;
 	char		host[64];
+	const char	*hostname; /* can be a hostname or IP address */
 	enum redis_role	role;
 
 	redisContext	*ctx;
@@ -54,14 +55,16 @@ struct redis {
 	int		state;
 	int		no_redis_required;
 	int		consecutive_errors;
-	time_t	restore_tick;
+	int64_t		restore_tick_us;
 	int		current_db;
 
-	struct event_base        *async_ev;
-	struct redisAsyncContext *async_ctx;
-	mutex_t                   async_lock;
-	GQueue                    async_queue;
-	int                       async_last;
+	struct event_base		*async_ev;
+	struct redisAsyncContext	*async_ctx;
+	mutex_t				async_lock;
+	GQueue				async_queue;
+	int64_t				async_last;
+
+	bool		update_resolve;
 };
 
 struct redis_hash {
@@ -81,14 +84,11 @@ extern struct redis		*rtpe_redis_write_disabled;
 extern struct redis		*rtpe_redis_notify;
 
 
-
-#define rlog(l, x...) ilog(l | LOG_FLAG_RESTORE, x)
-
 void redis_notify_loop(void *d);
 void redis_delete_async_loop(void *d);
 
 
-struct redis *redis_new(const endpoint_t *, int, const char *, enum redis_role, int);
+struct redis *redis_new(const endpoint_t *, int, const char *, const char *, enum redis_role, int, bool);
 struct redis *redis_dup(const struct redis *r, int db);
 void redis_close(struct redis *r);
 int redis_restore(struct redis *, bool foreign, int db);
@@ -97,46 +97,8 @@ void redis_delete(call_t *, struct redis *);
 void redis_wipe(struct redis *);
 int redis_async_event_base_action(struct redis *r, enum event_base_action);
 int redis_notify_subscribe_action(struct redis *r, enum subscribe_action action, int keyspace);
-int redis_set_timeout(struct redis* r, int timeout);
+int redis_set_timeout(struct redis* r, int64_t timeout);
 int redis_reconnect(struct redis* r);
 
 
-
-#define define_get_type_format(name, type)									\
-	static int redis_hash_get_ ## name ## _v(type *out, const struct redis_hash *h, const char *f,		\
-			va_list ap)										\
-	{													\
-		char key[64];											\
-														\
-		vsnprintf(key, sizeof(key), f, ap);								\
-		return redis_hash_get_ ## name(out, h, key);							\
-	}													\
-	static int redis_hash_get_ ## name ## _f(type *out, const struct redis_hash *h, const char *f, ...) {	\
-		va_list ap;											\
-		int ret;											\
-														\
-		va_start(ap, f);										\
-		ret = redis_hash_get_ ## name ## _v(out, h, f, ap);						\
-		va_end(ap);											\
-		return ret;											\
-	}
-
-#define define_get_int_type(name, type, func)								\
-	static int redis_hash_get_ ## name(type *out, const struct redis_hash *h, const char *k) {	\
-		str* s;										\
-													\
-		s = g_hash_table_lookup(h->ht, k);							\
-		if (!s)											\
-			return -1;									\
-		*out = func(s->s, NULL, 10);								\
-		return 0;										\
-	}
-
-
-
-
-
-
-
-
 #endif
