@@ -11,6 +11,7 @@
 #include <sys/resource.h>
 #include <unistd.h>
 #include <sys/syscall.h>
+#include <sys/time.h>
 
 
 #define THREAD_BUF_SIZE		64
@@ -37,21 +38,55 @@ struct rtpengine_common_config {
 	gboolean io_uring;
 	int io_uring_buffers;
 	int max_log_line_length;
+	int mos_type;				// enum in codec_def_t
 	char *evs_lib_path;
 	char *codec_chain_lib_path;
 	int codec_chain_runners;
 	int codec_chain_concurrency;
 	int codec_chain_async;
+	int codec_chain_opus_application;
 	int codec_chain_opus_complexity;
 };
 
 extern struct rtpengine_common_config *rtpe_common_config_ptr;
 
+INLINE unsigned int c_str_hash(const char *s);
+INLINE gboolean c_str_equal(const char *a, const char *b);
+
+TYPED_GHASHTABLE(charp_ht, char, char, c_str_hash, c_str_equal, g_free, g_free)
+
+union rtpenging_config_callback_arg {
+	charp_ht ht;
+	struct interface_config_callback_arg *icca;
+	struct transcode_config_callback_arg *tcca;
+} __attribute__((__transparent_union__));
+
+struct rtpenging_config_callback {
+	enum {
+		RCC_END = 0,
+		RCC_SECTION_KEYS,
+		RCC_FILE_GROUPS,
+	} type;
+	union rtpenging_config_callback_arg arg;
+	union {
+		struct {
+			char * const *name;
+			void (*callback)(const char *key, char *value,
+					union rtpenging_config_callback_arg);
+		} section_keys;
+		struct {
+			char * const *prefix;
+			void (*callback)(const char *name, charp_ht,
+					union rtpenging_config_callback_arg);
+		} file_groups;
+	};
+};
+
 
 
 /*** GLOBALS ***/
 
-extern __thread struct timeval rtpe_now;
+extern __thread int64_t rtpe_now;
 extern volatile bool rtpe_shutdown;
 
 
@@ -64,17 +99,22 @@ void resources(void);
 void wpidfile(void);
 void service_notify(const char *message);
 void config_load_free(struct rtpengine_common_config *);
-void config_load(int *argc, char ***argv, GOptionEntry *entries, const char *description,
+void config_load_ext(int *argc, char ***argv, GOptionEntry *entries, const char *description,
 		char *default_config, char *default_section,
-		struct rtpengine_common_config *);
+		struct rtpengine_common_config *,
+		const struct rtpenging_config_callback *);
+INLINE void config_load(int *argc, char ***argv, GOptionEntry *entries, const char *description,
+		char *default_config, char *default_section,
+		struct rtpengine_common_config *cc)
+{
+	config_load_ext(argc, argv, entries, description, default_config, default_section, cc, NULL);
+}
 
 char *get_thread_buf(void);
 int thread_create(void *(*func)(void *), void *arg, bool joinable, pthread_t *handle, const char *name);
 
 unsigned int in6_addr_hash(const void *p);
 int in6_addr_eq(const void *a, const void *b);
-unsigned int uint32_hash(const void *p);
-int uint32_eq(const void *a, const void *b);
 int num_cpu_cores(int);
 
 
@@ -82,8 +122,6 @@ int num_cpu_cores(int);
 
 #define ZERO(x)			memset(&(x), 0, sizeof(x))
 
-#define UINT64F			"%" G_GUINT64_FORMAT
-
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(char, g_free)
 typedef char *char_p;
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(char_p, g_strfreev)
@@ -104,7 +142,9 @@ INLINE unsigned int c_str_hash(const char *s) {
 INLINE gboolean c_str_equal(const char *a, const char *b) {
 	return g_str_equal(a, b);
 }
-
+INLINE void add_c_str_to_ht(const char *key, char *value, charp_ht ht) {
+	t_hash_table_insert(ht, g_strdup(key), value); // hash table takes ownership of both
+}
 
 /*** MUTEX ABSTRACTION ***/
 
@@ -124,7 +164,9 @@ typedef pthread_cond_t cond_t;
 #define rwlock_lock_r(l) __debug_rwlock_lock_r(l, __FILE__, __LINE__)
 #define rwlock_unlock_r(l) __debug_rwlock_unlock_r(l, __FILE__, __LINE__)
 #define rwlock_lock_w(l) __debug_rwlock_lock_w(l, __FILE__, __LINE__)
+#define rwlock_trylock_w(l) __debug_rwlock_trylock_w(l, __FILE__, __LINE__)
 #define rwlock_unlock_w(l) __debug_rwlock_unlock_w(l, __FILE__, __LINE__)
+#define RWLOCK_STATIC_INIT PTHREAD_RWLOCK_INITIALIZER
 
 #define cond_init(c) __debug_cond_init(c, __FILE__, __LINE__)
 #define cond_wait(c,m) __debug_cond_wait(c,m, __FILE__, __LINE__)
@@ -133,10 +175,10 @@ typedef pthread_cond_t cond_t;
 #define cond_broadcast(c) __debug_cond_broadcast(c, __FILE__, __LINE__)
 #define COND_STATIC_INIT PTHREAD_COND_INITIALIZER
 
-INLINE int __cond_timedwait_tv(cond_t *c, mutex_t *m, const struct timeval *tv) {
+INLINE int __cond_timedwait_tv(cond_t *c, mutex_t *m, int64_t tv) {
 	struct timespec ts;
-	ts.tv_sec = tv->tv_sec;
-	ts.tv_nsec = tv->tv_usec * 1000;
+	ts.tv_sec = tv / 1000000;
+	ts.tv_nsec = (tv % 1000000) * 1000;
 	return pthread_cond_timedwait(c, m, &ts);
 }
 
@@ -170,6 +212,7 @@ INLINE int __cond_timedwait_tv(cond_t *c, mutex_t *m, const struct timeval *tv)
 #define __debug_rwlock_lock_r(l, F, L) pthread_rwlock_rdlock(l)
 #define __debug_rwlock_unlock_r(l, F, L) pthread_rwlock_unlock(l)
 #define __debug_rwlock_lock_w(l, F, L) pthread_rwlock_wrlock(l)
+#define __debug_rwlock_trylock_w(l, F, L) pthread_rwlock_trywrlock(l)
 #define __debug_rwlock_unlock_w(l, F, L) pthread_rwlock_unlock(l)
 
 #define __debug_cond_init(c, F, L) pthread_cond_init(c, NULL)
@@ -234,6 +277,13 @@ INLINE int __debug_rwlock_lock_w(rwlock_t *m, const char *file, unsigned int lin
 	write_log(LOG_DEBUG, "rwlock_lock_w(%p) at %s:%u returning %i", m, file, line, ret);
 	return ret;
 }
+INLINE int __debug_rwlock_trylock_w(rwlock_t *m, const char *file, unsigned int line) {
+	int ret;
+	write_log(LOG_DEBUG, "rwlock_trylock_w(%p) at %s:%u ...", m, file, line);
+	ret = pthread_rwlock_trywrlock(m);
+	write_log(LOG_DEBUG, "rwlock_trylock_w(%p) at %s:%u returning %i", m, file, line, ret);
+	return ret;
+}
 INLINE int __debug_rwlock_unlock_r(rwlock_t *m, const char *file, unsigned int line) {
 	write_log(LOG_DEBUG, "rwlock_unlock_r(%p) at %s:%u", m, file, line);
 	return pthread_rwlock_unlock(m);
@@ -278,8 +328,8 @@ INLINE rwlock_w_lock_t *rwlock_auto_lock_w(rwlock_t *m) {
 }
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC(mutex_lock_t, mutex_ptr_unlock)
-G_DEFINE_AUTOPTR_CLEANUP_FUNC(rwlock_r_lock_t, rwlock_ptr_unlock_w)
-G_DEFINE_AUTOPTR_CLEANUP_FUNC(rwlock_w_lock_t, rwlock_ptr_unlock_r)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(rwlock_r_lock_t, rwlock_ptr_unlock_r)
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(rwlock_w_lock_t, rwlock_ptr_unlock_w)
 
 #define CONCAT2(a, b) a ## b
 #define CONCAT(a, b) CONCAT2(a, b)
@@ -295,59 +345,32 @@ G_DEFINE_AUTOPTR_CLEANUP_FUNC(rwlock_w_lock_t, rwlock_ptr_unlock_r)
 
 /*** TIMEVAL FUNCTIONS ***/
 
-INLINE long long timeval_us(const struct timeval *t) {
-	return (long long) ((long long) t->tv_sec * 1000000LL) + t->tv_usec;
+__attribute__((warn_unused_result))
+INLINE int64_t timeval_us(const struct timeval t) {
+	return ((int64_t) t.tv_sec * 1000000L) + t.tv_usec;
 }
-INLINE void timeval_from_us(struct timeval *t, long long ms) {
-	t->tv_sec = ms/1000000LL;
-	t->tv_usec = ms%1000000LL;
+INLINE int64_t timespec_us(const struct timespec t) {
+	return ((int64_t) t.tv_sec * 1000000L) + t.tv_nsec / 1000L;
 }
-INLINE long long timeval_diff(const struct timeval *a, const struct timeval *b) {
-	return timeval_us(a) - timeval_us(b);
+__attribute__((warn_unused_result))
+INLINE int64_t now_us(void) {
+	struct timeval now;
+	gettimeofday(&now, NULL);
+	return timeval_us(now);
 }
-INLINE void timeval_subtract(struct timeval *result, const struct timeval *a, const struct timeval *b) {
-	timeval_from_us(result, timeval_diff(a, b));
+__attribute__((warn_unused_result))
+INLINE struct timeval timeval_from_us(int64_t ms) {
+	return (struct timeval) { .tv_sec = ms/1000000LL, .tv_usec = ms%1000000LL };
 }
-INLINE void timeval_multiply(struct timeval *result, const struct timeval *a, const long multiplier) {
-	timeval_from_us(result, timeval_us(a) * multiplier);
-}
-INLINE void timeval_divide(struct timeval *result, const struct timeval *a, const long divisor) {
-	if (divisor == 0) {
-		result->tv_sec = 0;
-		result->tv_usec = 0;
-		return ;
-	}
-	timeval_from_us(result, timeval_us(a) / divisor);
-}
-INLINE void timeval_add(struct timeval *result, const struct timeval *a, const struct timeval *b) {
-	timeval_from_us(result, timeval_us(a) + timeval_us(b));
-}
-INLINE void timeval_add_usec(struct timeval *tv, long usec) {
-	timeval_from_us(tv, timeval_us(tv) + usec);
-}
-INLINE int long_cmp(long long a, long long b) {
-	if (a == b)
-		return 0;
-	if (a < b)
-		return -1;
-	return 1;
-}
-INLINE int timeval_cmp(const struct timeval *a, const struct timeval *b) {
-	int r = long_cmp(a->tv_sec, b->tv_sec);
-	if (r != 0)
-		return r;
-	return long_cmp(a->tv_usec, b->tv_usec);
-}
-// as a GCompareFunc
-int timeval_cmp_zero(const void *a, const void *b);
-int timeval_cmp_ptr(const void *a, const void *b);
-
-INLINE void timeval_lowest(struct timeval *l, const struct timeval *n) {
-	if (!n->tv_sec)
-		return;
-	if (!l->tv_sec || timeval_cmp(l, n) == 1)
-		*l = *n;
+__attribute__((warn_unused_result))
+INLINE int64_t timeval_lowest(const int64_t l, const int64_t n) {
+	if (!n)
+		return l;
+	if (!l || l > n)
+		return n;
+	return l;
 }
+__attribute__((warn_unused_result))
 INLINE double ntp_ts_to_double(uint32_t whole, uint32_t frac) {
 	return (double) whole + (double) frac / 4294967296.0;
 }
@@ -382,7 +405,7 @@ struct rtpe_g_tree_find_helper {
 	};
 };
 
-INLINE void *g_tree_find_first(GTree *t, GEqualFunc f, void *data) {
+INLINE void *rtpe_g_tree_find_first(GTree *t, GEqualFunc f, void *data) {
 	struct rtpe_g_tree_find_helper h = {
 		.func = f,
 		.data = data,
@@ -390,7 +413,15 @@ INLINE void *g_tree_find_first(GTree *t, GEqualFunc f, void *data) {
 	g_tree_foreach(t, rtpe_tree_find_first_cmp, &h);
 	return h.out_p;
 }
-INLINE void g_tree_find_all(GQueue *out, GTree *t, GEqualFunc f, void *data) {
+INLINE void *rtpe_g_tree_first(GTree *t) {
+#if GLIB_CHECK_VERSION(2,68,0)
+	GTreeNode *n = g_tree_node_first(t);
+	return n ? g_tree_node_value(n) : NULL;
+#else
+	return rtpe_g_tree_find_first(t, NULL, NULL);
+#endif
+}
+INLINE void rtpe_g_tree_find_all(GQueue *out, GTree *t, GEqualFunc f, void *data) {
 	struct rtpe_g_tree_find_helper h = {
 		.func = f,
 		.data = data,
@@ -398,26 +429,26 @@ INLINE void g_tree_find_all(GQueue *out, GTree *t, GEqualFunc f, void *data) {
 	};
 	g_tree_foreach(t, rtpe_tree_find_all_cmp, &h);
 }
-INLINE void g_tree_get_values(GQueue *out, GTree *t) {
-	g_tree_find_all(out, t, NULL, NULL);
+INLINE void rtpe_g_tree_get_values(GQueue *out, GTree *t) {
+	rtpe_g_tree_find_all(out, t, NULL, NULL);
 }
-INLINE void g_tree_find_remove_all(GQueue *out, GTree *t) {
+INLINE void rtpe_g_tree_find_remove_all(GQueue *out, GTree *t) {
 	GList *l;
 	g_queue_init(out);
-	g_tree_find_all(out, t, NULL, NULL);
+	rtpe_g_tree_find_all(out, t, NULL, NULL);
 	for (l = out->head; l; l = l->next)
 		g_tree_remove(t, l->data);
 }
-INLINE void g_tree_insert_coll(GTree *t, gpointer key, gpointer val, void (*cb)(gpointer, gpointer)) {
+INLINE void rtpe_g_tree_insert_coll(GTree *t, gpointer key, gpointer val, void (*cb)(gpointer, gpointer)) {
 	gpointer old = g_tree_lookup(t, key);
 	if (old)
 		cb(old, val);
 	g_tree_insert(t, key, val);
 }
-INLINE void g_tree_add_all(GTree *t, GQueue *q, void (*cb)(gpointer, gpointer)) {
+INLINE void rtpe_g_tree_add_all(GTree *t, GQueue *q, void (*cb)(gpointer, gpointer)) {
 	GList *l;
 	for (l = q->head; l; l = l->next)
-		g_tree_insert_coll(t, l->data, l->data, cb);
+		rtpe_g_tree_insert_coll(t, l->data, l->data, cb);
 	g_queue_clear(q);
 }
 
@@ -440,13 +471,6 @@ INLINE gboolean g_hash_table_steal_extended(GHashTable *ht, gconstpointer lookup
 }
 #endif
 
-#if !(GLIB_CHECK_VERSION(2,60,0))
-INLINE void g_queue_clear_full(GQueue *q, GDestroyNotify free_func) {
-	void *p;
-	while ((p = g_queue_pop_head(q)))
-		free_func(p);
-}
-#endif
 
 
 /*** MISC ***/
@@ -607,8 +631,19 @@ INLINE double atomic64_div(const atomic64 *n, const atomic64 *d) {
 }
 
 #define atomic_get_na(x) __atomic_load_n(x, __ATOMIC_RELAXED)
+#define atomic_get(x) __atomic_load_n(x, __ATOMIC_SEQ_CST)
 #define atomic_set_na(x,y) __atomic_store_n(x, y, __ATOMIC_RELAXED)
-#define atomic_inc_na(x) __atomic_fetch_add(x, 1, __ATOMIC_RELAXED);
+#define atomic_set(x,y) __atomic_store_n(x, y, __ATOMIC_SEQ_CST)
+#define atomic_inc_na(x) __atomic_fetch_add(x, 1, __ATOMIC_RELAXED)
+#define atomic_inc(x) __atomic_fetch_add(x, 1, __ATOMIC_SEQ_CST)
+#define atomic_dec(x) __atomic_fetch_sub(x, 1, __ATOMIC_SEQ_CST)
+#define atomic_add(x,y) __atomic_fetch_add(x, y, __ATOMIC_SEQ_CST)
+#define atomic_add_na(x,y) __atomic_fetch_add(x, y, __ATOMIC_RELAXED)
+#define atomic_sub(x,y) __atomic_fetch_sub(x, y, __ATOMIC_SEQ_CST)
+#define atomic_sub_na(x,y) __atomic_fetch_sub(x, y, __ATOMIC_RELAXED)
+#define atomic_exchange(x,y) __atomic_exchange_n(x, y, __ATOMIC_SEQ_CST)
+#define atomic_exchange_na(x,y) __atomic_exchange_n(x, y, __ATOMIC_RELAXED)
+#define atomic_compare_exchange(x,y,z) __atomic_compare_exchange_n(x, y, z, false, __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST)
 
 
 /*** ATOMIC BITFIELD OPERATIONS ***/
